/*******************************************************************************
 *                            C++ Verimag
 *    
 *
 *------------------------------------------------------------------------------
 * 	            Distributed and Complex Systems - Gires
 *------------------------------------------------------------------------------
 * bip-time.h
 *------------------------------------------------------------------------------
 *  Time for BIP2 Engine implementation
 *  + class Time        : time management
 *------------------------------------------------------------------------------
 * Created by Jacques COMBAZ on Wed Jan 13 2010
 *
 ******************************************************************************/

#ifndef BIP_TIME_H
#define BIP_TIME_H

#include <iostream>

/* for compatibility with old generated code - to be removed */
#define TIME_MIN INT_MIN
#define TIME_MAX INT_MAX

enum Unit { NANOSECOND = 0, MILLISECOND = 1, SECOND = 2 };

namespace BIP {
  class Time {
    /*** Protected data members ***/
    protected:
      long long mTime;

    /*** Constructors & Dectructors ***/
    public:
      inline Time()                        { }
      inline Time(const Time&   t)         { mTime = t.mTime; } 
      inline Time(      long    t)         { mTime = (long long)(t); }
      inline Time(      int     t)         { mTime = (long long)(t); }
      inline Time(const double& t)         { mTime = (long long)(t); }
      inline Time(const long long& t)      { mTime = (long long)(t); }

    /*** Public data members ***/
    public:
      inline Time& operator= (const Time& t) { if (this != &t) { mTime = t.mTime; } return *this; }

      inline const Time operator-() const {
        Time r;
             if (*this == MAX) r = MIN;
        else if (*this == MIN) r = MAX;
        else   r.mTime = -mTime;
	return r;
      }
      inline Time& operator+=(const Time& t) {
        if (*this != MAX && *this != MIN) {
          if (t == MAX || t == MIN) *this = t;
          else mTime += t.mTime;
        }
        return *this;
      }
      inline Time& operator-=(const Time& t) {
        if (*this != MAX && *this != MIN) {
               if (t == MAX) *this = MIN;
          else if (t == MIN) *this = MAX;
          else mTime -= t.mTime;
        }
        return *this;
      }
      inline const Time operator+(const Time& t) const { Time r = *this; r += t; return r; }
      inline const Time operator-(const Time& t) const { Time r = *this; r -= t; return r; }

      inline bool  operator< (const Time& t) const { return mTime <  t.mTime; }
      inline bool  operator<=(const Time& t) const { return mTime <= t.mTime; }
      inline bool  operator> (const Time& t) const { return t <  *this; }
      inline bool  operator>=(const Time& t) const { return t <= *this; }
      inline bool  operator==(const Time& t) const { return mTime == t.mTime; }
      inline bool  operator!=(const Time& t) const { return !(*this == t); }

      inline       operator const double() { return double(mTime); }
      inline       operator const int   () { return    int(mTime); }
      inline       operator const long  () { return   long(mTime); }

      friend std::ostream&  operator<<(std::ostream &, const Time&);
      friend std::istream&  operator>>(std::istream &,       Time&);

      /*** Predefined Constant Values ***/
      static const Time MIN;
      static const Time MAX;
  };

  /*** Friend Operators ***/
  inline std::ostream&  operator<<(std::ostream &o, const Time& t) {
         if (t == BIP::Time::MAX) o << "_";
    else if (t == BIP::Time::MIN) o << "_";
    else                          o << (t.mTime / 1000) << "ms";
    return o;
  }
  inline std::istream&  operator>>(std::istream &i, Time& t) {
    i >> t.mTime;
    t.mTime *= 1000;
    return i;
  }
  inline Time min(Time& t, Time& u) {
    if (t <= u) return t;
    else        return u;
  }
  inline Time max(Time& t, Time& u) {
    if (t >= u) return t;
    else        return u;
  }
}

#endif

