#ifndef ATOM_H_
#define ATOM_H_

#include <vector>
#include <pthread.h>

class CtrlState;
class Port;
class Connector;
class Priority;
class Scheduler;

/***********************************
 * Class: Atom
 * Description: Atomic component 
 ***********************************/
class Atom : public Component {

  /*** Public data members ***/
  public:
    enum Status {BUSY = 0, READY = 1};

    typedef std::vector<Connector*> connectors_type;
    typedef std::vector<Priority*> priorities_type;

    connectors_type mLinks;      /* Atom links (connectors) */
    priorities_type mConditions;  /* atom conditions */
    CtrlState*             BIP_STATE;   /* current control-state */
    Scheduler*             mScheduler;  /* back pointer to scheduler */
    Status                 mStatus;     /* status wrt schduler */
    bool                   mSelected;   /* selected for execution */

  protected:
    pthread_t              mThread;
    pthread_cond_t         mResume;
    pthread_mutex_t        mMutex;

  /*** Constructoris & Dectructors ***/
  public:
    Atom(const char*, Compound*);
    virtual ~Atom();
 
  /*** Public Data-Access Methods ***/
  public:
    connectors_type& links()
    {
      return mLinks;
    } 

    inline Status status() const { return mStatus; }
    inline void setStatus(const Status status) { mStatus = status; }
    
    inline pthread_cond_t* resume() { return &mResume; }
    inline pthread_mutex_t* mutex() { return &mMutex; }

    inline bool selected() const    { return mSelected; }
    inline void setSelected(bool b) { mSelected = b; }

  /*** Public Utility Methods ***/
  public:
    virtual bool       isAtom() const     { return true; }
    virtual bool       isCompound() const { return false; }
            void       resetLink();      /* mark links for re-evaluation */
            void       addPort(Port *p); /*      */

  /*** Public Methods visible from appln. ***/
  public:
    virtual void setup(Scheduler*);
    virtual void launch();
    virtual void goBusy();
    virtual void goReady();
    virtual void run() {}
    Port* sync(const int, ...);

    static void applyPriority(Port*, bool, Port*, bool);

    virtual void dump(FILE* file) const;

  protected:
    static void* Run(void*);
    static int nThread;
};

class CtrlState {
  /*** Private data members ***/
  private:
    char*   mName;
  /*** Constructoris & Dectructors ***/
  public:
    CtrlState(const char* = NULL);
    ~CtrlState();

    inline char* name() { return mName; }
};

#endif /* ATOM_H_*/
