#ifndef PORT_H
#define PORT_H

class Atom;
class Connector;

/***************************
 * Class: Port
 * Description: port class
 **************************/
class Port {
  /*** Protected data members ***/
  protected:
    char*      mName;       /* Name of the port */
    Atom*      mAtomHolder; /* holder for atomic port */
    Connector* mConnHolder; /* holer for conn port */
    bool       mSynced;     /* the holder syncs it     */
    bool       mElected;    /* the scheduler elects it */
    bool       mConnected;  /* i.e. connected, else internal label */
    bool       mInternal;   /* if internal port */
    bool       mDominated;  /* dominated in a priority rule */

  /*** Constructors & Dectructors ***/
  public:
    Port(const char*, 
         Atom*);
    Port(const char*,
         Connector*);
    virtual ~Port();

  /*** Public Data-Access Methods ***/
  public:
    inline char*      name()        const          { return mName; }
    inline Atom*      atomHolder()  const          { return mAtomHolder; }
    inline Connector* connHolder()  const          { return mConnHolder; } 
    inline bool       synced()      const          { return mSynced; }
    inline void       setSynced(bool synced)       { mSynced = synced; }
    inline bool       elected()     const          { return mElected; }
    inline void       setElected(bool elected)     { mElected = elected; }
    inline bool       isConnected() const          { return mConnected; }
    inline void       setConnected()               { mConnected = true; }
    inline bool       isInternal()  const          { return mInternal; }
    inline void       setInternal()                { mInternal = true; }
    inline bool       dominated()   const          { return mDominated; }
    inline void       setDominated(bool dominated) { mDominated = dominated; }
    inline void       reset()                      { mSynced = false; mDominated = false; }

  /*** Public Utility Methods ***/
  public:
    void execute();
    void addLink(Connector*);
    bool holderReady();
    void holderGoBusy(int&);
    void triggerAtom();

  /*** Public Methods invoked from appln. ***/
  public:
    bool complete() const;
    bool guard();

};

#endif
