

#ifndef PRIORITY_H
#define PRIORITY_H

#include <vector>
#include <cstdio>

class Interaction;
class Compound;
class Atom;
class Connector;

/*******************************
 * Class: Priority
 * Description: 
 *******************************/
class Priority {
  public:
    enum Status {BUSY, READY};

    typedef std::vector<Atom*> atoms_type;

  /*** Protected data members ***/
  protected:
    Compound*    mHolder; /* holder compound */
    Interaction* mLow;    /* the inhibited   */
    Interaction* mHigh;   /* the inhibitor   */
    Status mStatus;
    atoms_type mSupport; // the atoms required to compute the guard    

  /*** Constructors & Dectructors ***/
  public:
    Priority(Compound*, Interaction*, Interaction*);
  virtual ~Priority();

  /*** Public Data-Access Methods ***/
  public:
    inline Compound*    holder() const { return mHolder; }
    inline Interaction* low()    const { return mLow; }
    inline Interaction* high()   const { return mHigh; }
    inline Status       status() const { return mStatus; }
    inline void setStatus(const Status status) { mStatus = status; }

  /*** Public Utility Methods ***/
  public:
    virtual void setup();
    virtual bool allAtomsReady() const;
    virtual void dump(FILE* file) const;

  /*** Public Methods invoked from appln. ***/
  public:
    virtual bool guard() const { return true; }
};

#endif
