#ifndef SCHEDULER_H
#define SCHEDULER_H

#include "structure.h"
#include <list>
#include <pthread.h>

class Compound;

class Scheduler {
  public:

    typedef std::list<Interaction*> interactions_type;
    typedef std::list<Atom*> atoms_type;

  private:
    Compound*         mSystem;      /* the system under test                */
    atoms_type mReady;       /* snapshot (list) of ready atoms       */
    interactions_type mPending;     /* list of pending interactions         */
    int               mAtoms;       /* total number of atoms in the system  */
    int               mReadyAtoms;  /* number of ready atoms                */
    char              mInfo[512];   /* various infos buffer                 */

    pthread_cond_t    mResume;      /* block thread on a condition variable */
    pthread_mutex_t   mMutex;       /* mutex variable                       */
    
    /* execution parameters */
    int               mIntQuiet;
    bool              mQuiet;       /* 1 => dont show scheduled interactions */
    bool              mSimLimit;    /* true => simulation run has a limit    */
    int               mMaxSimLimit; /* max no of steps(interactions) executed 
                                       by engine                             */
    int               mSimStep;     /* count of steps executed by engine     */

    int               mIntLazy;
    bool              mLazy;        /* flag marking a lazy oracle */
    int               mRandSeed;    /* seed for randomized execution         */
    FILE*             mOutFile;     /* contains simulation output trace      */

#ifdef PERF
    BIP_Clock         comp_clk;     /* clock measuring computation time      */
    int               n_running_atoms; /* no. of running atoms at any moment */
#endif

  public:
    Scheduler(Compound*); 
    virtual ~Scheduler();

    interactions_type& pending() { return mPending; } 

    void parseCmdLine(int argc, char** argv);  
    void setup();
    void launch();

    void run();
    void selectEnabled(atoms_type&);
    void putReady(Atom*);
    void getReady(atoms_type&);
    void getReadyAtoms(atoms_type&);
    void waitAllAtomsReady(atoms_type&);
    void execute(Connector*, Interaction*, atoms_type&);
    void reEvaluate(atoms_type&);
    void incrAtomCount() { mAtoms++; }
    inline int nAtoms() { return mAtoms; }

    void removeAtom(Atom*);    

    void loginfo(FILE*) const;
};

#endif
